@@ -1,19 +1,27 @@
 ---------------------------------------------------------------------------
 -- @author Uli Schlachter
 -- @copyright 2010 Uli Schlachter
--- @release v3.5.7
+-- @module gears
 ---------------------------------------------------------------------------
 
---- gears
 
 return
 {
     color = require("gears.color");
     debug = require("gears.debug");
     object = require("gears.object");
-    sort = require("gears.sort");
     surface = require("gears.surface");
     wallpaper = require("gears.wallpaper");
+    timer = require("gears.timer");
+    cache = require("gears.cache");
+    matrix = require("gears.matrix");
+    shape = require("gears.shape");
+    protected_call = require("gears.protected_call");
+    geometry = require("gears.geometry");
+    math = require("gears.math");
+    table = require("gears.table");
+    string = require("gears.string");
+    filesystem = require("gears.filesystem");
 }
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
