@@ -1,4 +1,23 @@
 ---------------------------------------------------------------------------
+-- This container scrolls its inner widget inside of the available space. An
+-- example usage would be a text widget that displays information about the
+-- currently playing song without using too much space for long song titles.
+--
+-- Please note that mouse events do not propagate to widgets inside of the
+-- scroll container. Also, if this widget is causing too high CPU usage, you can
+-- use @{set_fps} to make it update less often.
+-- @usage
+-- wibox.widget {
+--    layout = wibox.container.scroll.horizontal,
+--    max_size = 100,
+--    step_function = wibox.container.scroll.step_functions
+--                    .waiting_nonlinear_back_and_forth,
+--    speed = 100,
+--    {
+--        widget = wibox.widget.textbox,
+--        text = "This is a " .. string.rep("very, ", 10) ..  " very long text",
+--    },
+-- }
 -- @author Uli Schlachter (based on ideas from Saleur Geoffrey)
 -- @copyright 2015 Uli Schlachter
 -- @classmod wibox.container.scroll
@@ -8,11 +27,11 @@ local cache = require("gears.cache")
 local timer = require("gears.timer")
 local hierarchy = require("wibox.hierarchy")
 local base = require("wibox.widget.base")
+local gtable = require("gears.table")
 local lgi = require("lgi")
 local GLib = lgi.GLib
 
 local scroll = {}
-local scroll_mt = { __index = scroll }
 local _need_scroll_redraw
 
 -- "Strip" a context so that we can use it for our own drawing
@@ -108,7 +127,11 @@ local function calculate_info(self, context, width, height)
 
         local x, y = 0, 0
         local function get_scroll_offset(size, visible_size)
-            return self._private.step_function(self._private.timer:elapsed(), size, visible_size, self._private.speed, self._private.extra_space)
+            return self._private.step_function(self._private.timer:elapsed(),
+                                               size,
+                                               visible_size,
+                                               self._private.speed,
+                                               self._private.extra_space)
         end
         if self._private.dir == "h" then
             x = -get_scroll_offset(surface_width - extra, width)
@@ -380,7 +403,7 @@ local function get_layout(dir, widget, fps, speed, extra_space, expand, max_size
     ret._private.timer = GLib.Timer()
     ret._private.scroll_timer = nil
 
-    setmetatable(ret, scroll_mt)
+    gtable.crush(ret, scroll, true)
 
     ret:set_direction(dir)
     ret:set_widget(widget)
